'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cssRule = exports.style = undefined;

var _electron = require('electron');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// --- Helpers ---

/** Hide elements by a selector */
var hide = function hide(elementSelector) {
  var kill = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

  var nodeList = document.querySelectorAll(elementSelector);
  _lodash2.default.forEach(nodeList, function (node) {
    var element = node;
    element.style.display = 'none';
    if (kill) {
      element.remove();
    }
  });
};

/** Set CSS style by a selector */
var style = exports.style = function style(elementSelector, styleObject) {
  var nodeList = document.querySelectorAll(elementSelector);
  _lodash2.default.forEach(nodeList, function (node) {
    var element = node;
    _lodash2.default.forIn(styleObject, function (value, key) {
      element.style[key] = value;
    });
  });
};

/** Inject a CSS rule to the page (in a <style> tag) */
var cssRule = exports.cssRule = function cssRule(styles) {
  var tag = document.createElement('style');
  tag.type = 'text/css';
  tag.appendChild(document.createTextNode(styles));
  document.head.appendChild(tag);
};

// --- UI modifications ---
function _redirectButton(button, URL, reverseURLChange) {
  if (button) {
    button.addEventListener('click', function (e) {
      _electron.remote.shell.openExternal(URL);
      if (reverseURLChange) setImmediate(history.back);
      e.preventDefault();
      return false;
    });
  }
}

/** Change the Shop button to open Shop in external browser */
function fixShopButton() {
  _redirectButton(document.querySelector('[data-type="shop"]'), 'https://play.google.com/store/music?feature=music_general');
}

function handleSubscribeButton() {
  _redirectButton(document.querySelector('.sub[data-type="sub"]'), 'https://play.google.com/music/listen#/sulp', true);
}

/** Hide buttons & elements that don't work */
function hideNotWorkingStuff() {
  // Top left account control buttons
  hide('#material-one-right #gb > div > div > div:not(:last-child)');
  style('#material-one-right #gb > div > div > div:last-child', { display: 'block', float: 'right' });
  style('#material-one-right #gb > div > div', { display: 'block', float: 'right' });

  // Built in mini player buttons
  hide('.player-top-right-items > paper-icon-button');

  // Settings options that won't work
  hide('[data-action="upload-music"]');
  hide('[data-action="help-and-feedback"]');
  hide('[data-action="send-gift"]');

  // Hide the upload music button in settings
  cssRule('.music-sources-card.settings-card {display: none !important}');

  hide('.upload-dialog-bg', true);
  hide('.upload-dialog', true);

  setInterval(function () {
    return hide('.goog-menu.now-playing-menu > .goog-menuitem:nth-child(3)');
  }, 500);
}

/** Create the Desktop Settings button in the left sidebar */
function installDesktopSettingsButton() {
  var dSettings = document.createElement('a');
  dSettings.setAttribute('data-type', 'desktopsettings');
  dSettings.setAttribute('class', 'nav-item-container tooltip');
  dSettings.setAttribute('href', '');
  dSettings.setAttribute('no-focus', '');
  dSettings.innerHTML = '<iron-icon icon="settings" alt="" class="x-scope iron-icon-1"></iron-icon>Desktop settings'; // eslint-disable-line
  dSettings.addEventListener('click', function (e) {
    Emitter.fire('window:settings');
    e.preventDefault();
    e.stopPropagation();
    return false;
  });
  document.querySelectorAll('.nav-section.material')[0].insertBefore(dSettings, document.querySelectorAll('.nav-section.material > a')[2]); // eslint-disable-line
}

/* eslint-disable max-len, no-multi-str */
function installNowPlayingButton(text, id, callback) {
  var ripple = '<paper-ripple style="-webkit-user-select: none;">\n                  <div id="background" class="style-scope paper-ripple" style="-webkit-user-select: none;"></div>\n                  <div id="waves" class="style-scope paper-ripple" style="-webkit-user-select: none;"></div>\n                  </paper-ripple>';
  var content = document.createElement('div');
  content.setAttribute('class', 'goog-menuitem-content');
  content.innerHTML = ripple + ' ' + text;

  var button = document.createElement('div');
  button.setAttribute('class', 'goog-menuitem');
  button.setAttribute('role', 'menuitem');
  button.setAttribute('id', id);
  button.appendChild(content);

  var nowPlayingMenu = document.querySelector('.goog-menu.song-menu');
  nowPlayingMenu.appendChild(button);

  button.addEventListener('click', function (e) {
    // DEV: Hacky but smooth way to close menu when clicked.
    setTimeout(function () {
      document.querySelector('.goog-menu.song-menu').style.display = 'none';
    }, 150);
    callback(e);
  });
}
/* eslint-enable max-len */

function installNowPlayingSeperator() {
  var seperator = document.createElement('div');
  seperator.setAttribute('role', 'menuitem');
  seperator.setAttribute('class', 'goog-menuseparator');

  var nowPlayingMenu = document.querySelector('.goog-menu.song-menu');
  nowPlayingMenu.appendChild(seperator);
}

function installNowPlayingMenu() {
  installNowPlayingSeperator();
  installNowPlayingButton('Pause after this song', ':gpmdppause', function () {
    Emitter.fireAtGoogle('pauseAfter:show');
  });
  installNowPlayingSeperator();
  installNowPlayingButton('Show Lyrics (Beta)', ':gpmdplyrics', function () {
    Emitter.fireAtMain('lyrics:show');
  });
}

function handleZoom() {
  var zoom = Settings.get('zoom', 1);
  _electron.remote.getCurrentWebContents().setZoomFactor(zoom);
  window.addEventListener('keydown', function (e) {
    if (!(e.ctrlKey || e.metaKey) || e.repeat) return;
    var webContents = _electron.remote.getCurrentWebContents();
    if (e.which === 189) {
      // Zoom out
      zoom -= 0.1;
    } else if (e.which === 187) {
      // Zoom in
      zoom += 0.1;
    } else if (e.which === 48) {
      zoom = 1;
    } else {
      return;
    }
    webContents.setZoomFactor(zoom);
    Emitter.fire('settings:set', {
      key: 'zoom',
      value: zoom
    });
  });
}

// Modify the GUI after everything is sufficiently loaded
window.wait(function () {
  hideNotWorkingStuff();
  fixShopButton();
  handleSubscribeButton();
  installDesktopSettingsButton();
  handleZoom();
  installNowPlayingMenu();
});